#!/bin/bash

# This example shows how to modify which files vhistify documents or does
# not document. You can use the environment variables
#
# VHISTIFY_INCLUDE_DIRS
#
# and
#
# VHISTIFY_EXCLUDE_DIRS
#
# to change vhistify's behaviour. By default, vhistify ignores files in /tmp.
# This example sets VHISTIFY_INCLUDE_DIRS to add files in a sub directory
# of /tmp back into the generated VHIST file.
# 
# THE FIRST LINES OF THIS FILE TEST THAT ALL PROGRAMS AND FILES NEEDED BY
# THIS EXAMPLE ARE AVAILABLE. THESE LINES ARE NOT PART OF THE EXAMPLE.
# YOU CAN JUST IGNORE THEM. THE EXAMPLE BEGINS BELOW THIS SECTION.

set -e # enable error checking
source ../common/tools.bash

# check if everything is set up correctly
check-if-program-exists vhistify
check-if-program-exists echo
check-if-program-exists mktemp
check-if-program-exists cp

# ------------ EXAMPLE BEGINS HERE ----------------

# by default, vhistify ignores /tmp
# this example shows how to document files in /tmp anyway

export VHISTIFY_RENAME_OLD_VHISTFILES=true

# 1) create a directory in /tmp
MYTEMPDIR=$( mktemp -d )

# 2) tell vhistify to include files in this dir
export VHISTIFY_INCLUDE_DIRS="$MYTEMPDIR"

# 3) create a file in the tempdir.
echo "hello world" > "$MYTEMPDIR/test.dat"

# 4) do something with this file
vhistify -- cp "$MYTEMPDIR/test.dat" .

# ------------- EXAMPLE ENDS HERE ------------------

