#!/bin/bash

# This example shows how to document FSL (FMRIB Software Library) tools with
# vhistify. The example executes several FSL tools and the complete workflow
# is documented with VHIST.
# 
# THE FIRST LINES OF THIS FILE TEST THAT ALL PROGRAMS AND FILES NEEDED BY
# THIS EXAMPLE ARE AVAILABLE. THESE LINES ARE NOT PART OF THE EXAMPLE.
# YOU CAN JUST IGNORE THEM. THE EXAMPLE BEGINS BELOW THIS SECTION.

set -e # enable error checking
source ../common/tools.bash

# check if everything is set up correctly
check-if-program-exists vhistify
check-if-program-exists setup_fsl.sh
check-if-program-exists curl

# get demo data from the server
download-demodata

# ------------ EXAMPLE BEGINS HERE ----------------

export VHISTIFY_RENAME_OLD_VHISTFILES=true
export VHISTIFY_PLUGINS=fsl
# don't embed NIfTI files
export VHISTIFY_DONT_EMBED_PATTERNS='*.nii.gz,*.nii'

export SOURCE_DIR=../demodata

# 1) initialize FSL
source setup_fsl.sh

# 2) extract brain from MRI
vhistify --title="Extract Brain" -- bet "$SOURCE_DIR/pat_mmm_mri.nii.gz" pat_mmm_mri-stripped.nii.gz -m -v

# 3) coregister MRI onto PET and store the transformation
vhistify --title="Coregister Images" -- flirt -in pat_mmm_mri-stripped.nii.gz -ref "$SOURCE_DIR/pat_mmm_fmz.nii.gz" -omat "mr->pet.mat" -v

# 4) segment MRI into grey matter, white matter, csf
vhistify --title="Segment Brain" -- fast -v pat_mmm_mri-stripped.nii.gz

# 5) transform segmentation onto PET image
vhistify --title="Apply Transformation" -- flirt -in "pat_mmm_mri-stripped_seg.nii.gz" -ref "$SOURCE_DIR/pat_mmm_fmz.nii.gz" -applyxfm -init "mr->pet.mat" -out segmented_pet.nii.gz -v -interp nearestneighbour

# ------------- EXAMPLE ENDS HERE ------------------

