#!/bin/bash

# This example shows how to disable the creation of md5 checksums for large 
# files. It also shows how vhistify can reuse md5 checksums created by your
# application.
#
# THE FIRST LINES OF THIS FILE TEST THAT ALL PROGRAMS AND FILES NEEDED BY
# THIS EXAMPLE ARE AVAILABLE. THESE LINES ARE NOT PART OF THE EXAMPLE.
# YOU CAN JUST IGNORE THEM. THE EXAMPLE BEGINS BELOW THIS SECTION.

set -e # enable error checking
source ../common/tools.bash

# check if everything is set up correctly
check-if-program-exists vhistify
check-if-program-exists dd
check-if-program-exists md5sum

# ------------ EXAMPLE BEGINS HERE ----------------

export VHISTIFY_RENAME_OLD_VHISTFILES=true
export VHISTIFY_PLUGINS=nf,largefile

# 1) create a large file with random contents during the first execution of
#    this example
RANDOM_FILE=random.data
if [ ! -e "$RANDOM_FILE" ];
then
    echo "creating a large random file at $RANDOM_FILE"
    echo "this might take some time..."
    dd if=/dev/urandom of="$RANDOM_FILE" count=500000; # 250 MB
fi

# 2) calculate the md5 sum by hand. The "largefile" plugin will disable md5
#    creation in vhistify.
vhistify --title="MD5 of large file" -- md5sum "$RANDOM_FILE"

# 3) store the md5 sum in a file. vhistify will reuse this md5 sum since the
#    "largefile" plugin is active. The md5 file must not contain anything
#    except the md5 checksum.
MD5_FILE="$RANDOM_FILE.md5"
md5sum "$RANDOM_FILE" | cut --bytes=1-32 > "$MD5_FILE"
vhistify --title="MD5 of large file" -- md5sum "$RANDOM_FILE"

# ------------- EXAMPLE ENDS HERE ------------------

