#!/bin/bash

# This example shows how to document Matlab/SPM scripts with vhisitify. The
# example executes a SPM script and documents it with VHIST. Make sure that
# spm is installed on your system.
# 
# THE FIRST LINES OF THIS FILE TEST THAT ALL PROGRAMS AND FILES NEEDED BY
# THIS EXAMPLE ARE AVAILABLE. THESE LINES ARE NOT PART OF THE EXAMPLE.
# YOU CAN JUST IGNORE THEM. THE EXAMPLE BEGINS BELOW THIS SECTION.

set -e # enable error checking
source ../common/tools.bash

# !!!!! change this variable to adjust the version of Matlab you want to use
MATLAB_EXECUTABLE=matlab_2011b

# check if everything is set up correctly
check-if-program-exists vhistify
check-if-program-exists curl
check-if-program-exists gunzip
check-if-program-exists "$MATLAB_EXECUTABLE"

# get demo data from the server and unzip it
download-demodata
gunzip -c ../demodata/pat_mmm_fmz.nii.gz > ../demodata/pat_mmm_fmz.nii

# ------------ EXAMPLE BEGINS HERE ----------------

export VHISTIFY_RENAME_OLD_VHISTFILES=true
export VHISTIFY_PLUGINS=matlab,spm,nf,previews

# !!!!! set the value of this variable to your SPM directory
export SPM_PATH=/daten/share/opt/spm8

# By default, Matlab reads code from stdin. We can prevent this by adding
# the --no-stdin option to vhistify
vhistify --title="SPM Test" --no-stdin -- "$MATLAB_EXECUTABLE" -nodesktop -nosplash -r 'spmTest'

# ------------- EXAMPLE ENDS HERE ------------------

