#!/bin/bash

# This plugin shows how to use stdin in your scripts when documenting them
# with vhistify. vhistify will not document stdin, but just pass it to your
# application. This means that you can pass passwords to your application
# and the passwords will not appear inside the VHIST file. It also means that
# the standard input should not be neccessary to understand what your program
# does. Otherwise, the generated documentation will be incomplete.
# 
# THE FIRST LINES OF THIS FILE TEST THAT ALL PROGRAMS AND FILES NEEDED BY
# THIS EXAMPLE ARE AVAILABLE. THESE LINES ARE NOT PART OF THE EXAMPLE.
# YOU CAN JUST IGNORE THEM. THE EXAMPLE BEGINS BELOW THIS SECTION.

set -e # enable errors
source ../common/tools.bash

# check if everything is set up correctly
check-if-program-exists vhistify
check-if-program-exists g++
check-if-program-exists python

# ------------ EXAMPLE BEGINS HERE ----------------

export VHISTIFY_RENAME_OLD_VHISTFILES=true

# 1) run C++ example
g++ hello.cpp -o hello
vhistify -- ./hello

# 2) run Python examples. Python buffers stdout when stdout is written to
# a pipe. This can cause problems when reading data from stdin (e.g. python
# does not show the question for the user, yet, but waits for user input)
# The first line flushes stdout explicitly in the Python script.
# The second line disables buffering in the Python plugin.
vhistify -- python hello.py
vhistify --plugins=python -- python hello2.py

# ------------- EXAMPLE ENDS HERE ------------------

