
import os
import itertools

import utils
import plugintools

def gen2dMatrix(d1, d2):
    """ Generates a d1xd2 matrix containing zeros (integer) """
    mat = []
    for i in range(d1):
        mat.append([0] * d2)
    return mat


def stringDistance(string1, string2):
    """ Implements Levenshtein distance to check the similarity of two
        strings """
    d1 = len(string1)
    d2 = len(string2)
    
    if d1 == 0 or d2 == 0:
        return max(d1, d2)
    
    d = gen2dMatrix(d1, d2) # distances
    for i, c1 in enumerate(string1):
        for j, c2 in enumerate(string2):
            d[i][j] = int(c1 != c2)
    
    # iterate over all matrix indices
    for i, j in itertools.product(range(d1), range(d2)):
        if i == 0:
            if j == 0:
                # upper left corner does not change
                pass
            else:
                # one border, skips chars in one string
                d[i][j] = 1 + d[i][j-1]
        else:
            if j == 0:
                # the other border, skips chars in the other string
                d[i][j] = 1 + d[i-1][j]
            else:
                # somewhere in the middle
                # we either consume a char in both strings (1. parameter),
                # or one in one string (2. and 3. parameter)
                d[i][j] = min(d[i][j] + d[i-1][j-1], 1 + d[i][j-1],
                        1 + d[i-1][j])
    
    # lower-right corner contains the distance between the string
    return d[-1][-1]


def getSupportedVars(plugins):
    supportedVars = ["VHISTIFY_PLUGINS", "VHISTIFY_DEFAULT_PLUGINS",
            "VHISTIFY_RENAME_OLD_VHISTFILES", "VHISTIFY_TITLE",
            "VHISTIFY_DESCRIPTION", "VHISTIFY_COMMENT"]
    
    for plugin in plugins:
        supportedVars.extend(plugin.getSupportedEnvironmentVariables())
    return supportedVars


def mightBeVhistifyVariable(var):
    if "_" not in var:
        return False
    else:
        # VHISTIFY_FOO => VHISTIFY
        prefix = var.split("_")[0]
        return stringDistance(prefix, "VHISTIFY") <= 1


def matchesAny(var, supportedVars):
    candidates = []
    for supportedVar in supportedVars:
        d = stringDistance(var, supportedVar)
        if d == 0:
            return True, []
        elif d <= 5: # close enough
            candidates.append(supportedVar)
    return False, candidates


def checkEnvironmentVariables(plugins=None, environmentVars=None):
    if plugins is None:
        plugins = plugintools.getAllPluginClasses()
    
    if environmentVars is None:
        environmentVars = os.environ.keys()
    
    supportedVars = getSupportedVars(plugins)
    
    for var in environmentVars:
        if not mightBeVhistifyVariable(var):
            continue
        
        found, candidates = matchesAny(var, supportedVars)
        if found:
            continue
        
        # seems to be a misspelled variable
        if len(candidates) > 0:
            msg = ("*** WARNING: unknown environment variable %s\n"
                    "Possible candidates are: %s")
            print msg % (var, utils.joinStrings(candidates))
        else:
            msg = "*** WARNING: unknown environment variable %s"
            print msg % var
        print ""

