
import os

class FileInfo(object):
    def __init__(self, filename, origFilename, fd, types = ()):
        self.filename = filename
        self.origFilename = origFilename
        self.fd = fd
        self.types = set(types)
    
    def addTypes(self, types):
        self.types.update(types)
    
    def toString(self):
        result = []
        result.append(self.filename)
        result.append("    " + str(self.types))
        return "\n".join(result)


def makeAbsolute(path, cwd):
    if not os.path.isabs(path):
        path = os.path.realpath(os.path.join(cwd, path))
    return path


def handleOpen(syscall, cwd):
    origFilename, flags = syscall.args
    fd = syscall.retval
    filename = makeAbsolute(origFilename, cwd)
    types = []
    if "O_RDONLY" in flags:
        types.append("in")
    if "O_WRONLY" in flags:
        types.append("out")
    if "O_RDWR" in flags and "O_TRUNC" in flags:
        types.append("out")
    if "O_RDWR" in flags and "O_TRUNC" not in flags:
        types.extend(["in", "out"])
    return FileInfo(filename, origFilename, fd, types)


def handleCreat(syscall, cwd):
    print "!!!"
    origFilename, permissions = syscall.args
    print origFilename, permissions
    fd = syscall.retval
    filename = makeAbsolute(origFilename, cwd)
    types = ["out"]
    return FileInfo(filename, origFilename, fd, types)


def addFile(finfo, files):
    if finfo.filename not in files:
        files[finfo.filename] = finfo
    else:
        files[finfo.filename].addTypes(finfo.types)


def createFileList(syscalls, cwd):
    files = {}
    programs = []
    
    for syscall in syscalls:
        if syscall.retval != None and syscall.retval < 0:
            continue
        if syscall.name == "open":
            finfo = handleOpen(syscall, cwd)
            addFile(finfo, files)
        elif syscall.name == "creat":
            finfo = handleCreat(syscall, cwd)
            addFile(finfo, files)
        elif syscall.name == "chdir":
            cwd = makeAbsolute(syscall.args[0], cwd)
        elif syscall.name == "rename":
            source = makeAbsolute(syscall.args[0], cwd)
            destination = makeAbsolute(syscall.args[1], cwd)
            if source in files:
                finfo = files[source]
                del files[source]
                finfo.filename = destination
                files[destination] = finfo
        elif syscall.name == "execve":
            executable = syscall.args[0]
            realExecutable = os.path.realpath(executable)
            programs.append((realExecutable, executable))


    files = files.values()
    return files, programs

