
import os
import sys
import utils

def findExecutable(paths, executables):
    """ Searches for the one of the given executables in the given list
    of paths and returns the first one, it finds. Search first for
    executables[0] at all paths, then executables[1] and so on. If it
    does not find any executable, the routine throws a RuntimeError.
    """
    checkedPaths = []
    for executable in executables:
        for path in paths:
            fullpath = os.path.join(path, executable)
            checkedPaths.append(fullpath)
            
            if os.path.exists(fullpath):
                return fullpath
    checkedPaths = ['"%s"' % p for p in checkedPaths]
    raise RuntimeError("could not find a vhistadd executable at the following "
            "locations: " + utils.joinStrings(checkedPaths))


def findVhistadd():
    """ This routine tries to find the vhistadd executable at a variety of
    places. On windows, ``findVhistadd`` will search for both,
    ``vhistadd.py`` and ``vhistadd.exe`` and it will prefer ``vhistadd.exe``.
    On all other systems, ``vhistadd.py`` is used. This routine tries to
    gracefully deal with frozen executables (py2exe), a zipped ``vhistadd``
    module and a normal, unzipped module.
    """
    # helper predicates
    isFrozen = getattr(sys, "frozen", "")
    isModuleInZipLibrary = not os.path.exists(__file__)
    isWindows = sys.platform == "win32"
    
    scriptDir = os.path.dirname(os.path.realpath(sys.argv[0]))
    moduleDir = os.path.dirname(os.path.abspath(__file__))
    repositoryDir = os.path.realpath(os.path.join(moduleDir,
            "../../core_py/bin"))
    searchPaths = []
    executables = []
    if isFrozen or isModuleInZipLibrary:
        searchPaths.extend([scriptDir, os.path.join(scriptDir, "vhistadd/bin")])
    else:
        searchPaths.extend([os.path.join(moduleDir, "vhistadd/bin"),
                repositoryDir])
    
    if isFrozen and isWindows:
        executables = ["vhistadd.exe"]
    elif isWindows:
        executables = ["vhistadd.exe", "vhistadd.py"]
    else:
        executables = ["vhistadd.py"]
    
    return findExecutable(searchPaths, executables)


def getCommandlinePrefix():
    """ Returns a list, which contains a commandline call to ``vhistadd``
    without any arguments. This list can be extended with additional
    commandline arguments to form a complete ``vhistadd`` call. You can
    use the %subprocess.Popen class to execute the commandline call.
    """
    vhistExecutable = findVhistadd()
    if vhistExecutable.endswith(".py"):
        return [sys.executable, vhistExecutable]
    else: # .exe
        return [vhistExecutable]

