
import datetime
import os
import glob

def readFile(filename):
    f = open(filename, "rb")
    content = f.read()
    f.close()
    return content


def writeFile(filename, content):
    f = open(filename, "wb")
    f.write(content)
    f.close()


def getDateString():
    curdate = datetime.datetime.now()
    datestr = curdate.strftime("%Y-%m-%d_%H-%M-%S")
    return datestr


def isSubdir(subdir, dirs):
    """
    checks if a 'subdir' is a sub directory of one of the directories in
    'dirs'
    """
    for dir in dirs:
        absDir = os.path.abspath(dir)
        absSubDir = os.path.abspath(subdir)
        if os.path.commonprefix([absDir, absSubDir]) == absDir:
            return True
    return False


def flatten(alist):
    return [item for sublist in alist for item in sublist]


def recursiveGlob(root, patterns, excludeDirs = None, excludePatterns = None):
    """
    Recursively walks over all subfolders of 'root', collecting files
    and directories which match one pattern in 'patterns'. Directories in
    'excludeDirs' are recursively ignored.
    """
    globs = set()
    if excludeDirs == None:
        excludeDirs = []
    excludeDirs = [os.path.join(root, d) for d in excludeDirs]
    
    if excludePatterns == None:
        excludePatterns = []
    dirs = os.walk(root)
    
    for (dir, subdirs, files) in dirs:
        if not ".svn" in dir and not isSubdir(dir, excludeDirs):
            for pattern in patterns:
                p = os.path.join(dir, pattern)
                globs = globs.union(glob.glob(p))
            
            for pattern in excludePatterns:
                p = os.path.join(dir, pattern)
                globs = globs.difference(glob.glob(p))
            
    return set(globs)


def joinStrings(alist, separator=', ', lastSeparator=' and '):
    if len(alist) == 0:
        return ""
    elif len(alist) == 1:
        return alist[0]
    else:
        firstPart = separator.join(alist[:-1])
        lastPart = alist[-1]
        return firstPart + lastSeparator + lastPart
