"""

This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: imageresources.py 2684 2013-04-16 14:24:21Z ahuesgen $
"""

import pdfobjs, pdftypes

FONT1_ID      = 7
FONT2_ID      = 8
FONT3_ID      = 9
FONT4_ID      = 10
FONT5_ID      = 11

FONT_OBJ = """\
<<
    /Type /Font
    /Subtype /Type1
    /Name %s
    /BaseFont %s
    /Encoding /WinAnsiEncoding
>>
"""

FONT1_OBJ = FONT_OBJ % ("/F1", "/Helvetica")
FONT2_OBJ = FONT_OBJ % ("/F2", "/Courier")
FONT3_OBJ = FONT_OBJ % ("/F3", "/Courier-Bold")
FONT4_OBJ = FONT_OBJ % ("/F4", "/Courier-Oblique")
FONT5_OBJ = FONT_OBJ % ("/F5", "/Courier-BoldOblique")


def genFonts():
    font1 = pdfobject.PDFSimpleObject(FONT1_ID, 0, FONT1_OBJ)
    font2 = pdfobject.PDFSimpleObject(FONT2_ID, 0, FONT2_OBJ)
    font3 = pdfobject.PDFSimpleObject(FONT3_ID, 0, FONT3_OBJ)
    font4 = pdfobject.PDFSimpleObject(FONT4_ID, 0, FONT4_OBJ)
    font5 = pdfobject.PDFSimpleObject(FONT5_ID, 0, FONT5_OBJ)
    return font1, font2, font3, font4, font5


def genBaseResources(id, gen = 0):
    font1, font2, font3, font4, font5 = genFonts()

    resObj = pdfobjs.PDFStructObject(id, gen)
    resObj.set("/ProcSet", ["/PDF", "/Text", "/ImageB"])
    resObj.set("/Font/F1", font1)
    resObj.set("/Font/F2", font2)
    resObj.set("/Font/F3", font3)
    resObj.set("/Font/F4", font4)
    resObj.set("/Font/F5", font5)
    return resObj


def genImageResources(imageObjects, id, gen = 0):
    resObj = genBaseResources(id, gen)
    for i, img in enumerate(imageObjects):
        resObj.set("/XObject/I%d" % (i+1), img)
    
    return resObj

