
import resources, pdfobjs, pdfadvobjs, pdftypes

# Fixed IDs in the PDF file
INFO_ID       = 1
CATALOG_ID    = 2
PAGES_ID      = 3
PAGELABELS_ID = 4
OUTLINES_ID   = 5
RESOURCES_ID  = 6
FONT1_ID      = resources.FONT1_ID
FONT2_ID      = resources.FONT2_ID
FONT3_ID      = resources.FONT3_ID
FONT4_ID      = resources.FONT4_ID
FONT5_ID      = resources.FONT5_ID
PAPERCLIP1_ID = 12
PAPERCLIP2_ID = 13
PAPERCLIP3_ID = 14

# References to fixed objects
INFO_REF       = "%d 0 R" % (INFO_ID)
CATALOG_REF    = "%d 0 R" % (CATALOG_ID)
PAGES_REF      = "%d 0 R" % (PAGES_ID)
PAGELABELS_REF = "%d 0 R" % (PAGELABELS_ID)
OUTLINES_REF   = "%d 0 R" % (OUTLINES_ID)
RESOURCES_REF  = "%d 0 R" % (RESOURCES_ID)
FONT1_REF      = "%d 0 R" % (FONT1_ID)
FONT2_REF      = "%d 0 R" % (FONT2_ID)
FONT3_REF      = "%d 0 R" % (FONT3_ID)
FONT4_REF      = "%d 0 R" % (FONT4_ID)
FONT5_REF      = "%d 0 R" % (FONT5_ID)
PAPERCLIP1_REF = "%d 0 R" % (PAPERCLIP1_ID)
PAPERCLIP2_REF = "%d 0 R" % (PAPERCLIP2_ID)
PAPERCLIP3_REF = "%d 0 R" % (PAPERCLIP3_ID)

# simple pdf objects
CATALOG_OBJ = """\
<<
    /Type /Catalog
    /Pages %s
    /PageLabels %s
    /Outlines %s
    /PageMode /UseOutlines
    /PageLayout /OneColumn
>>
""" % (PAGES_REF, PAGELABELS_REF, OUTLINES_REF)

RESOURCES_OBJ = """\
<<
    /ProcSet [/PDF /Text]
    /Font
    <<
        /F1 %s
        /F2 %s
        /F3 %s
        /F4 %s
        /F5 %s
    >>
>>
""" % (FONT1_REF, FONT2_REF, FONT3_REF, FONT4_REF, FONT5_REF)

FONT_OBJ = """\
<<
    /Type /Font
    /Subtype /Type1
    /Name %s
    /BaseFont %s
    /Encoding /WinAnsiEncoding
>>
"""

FONT1_OBJ = FONT_OBJ % ("/F1", "/Helvetica")
FONT2_OBJ = FONT_OBJ % ("/F2", "/Courier")
FONT3_OBJ = FONT_OBJ % ("/F3", "/Courier-Bold")
FONT4_OBJ = FONT_OBJ % ("/F4", "/Courier-Oblique")
FONT5_OBJ = FONT_OBJ % ("/F5", "/Courier-BoldOblique")

PAPERCLIP_STREAM = """\
%d w
0 J
0 j
65 30 m
65 75 l
65 80 55 95 50 95 c
45 95 35 80 35 75 c
35 10 l
35 7 37 5 40 5 c
56 5 l
59 5 61 7 61 10 c
61 65 l
61 80 39 80 39 65 c
39 30 l
S"""

PAPERCLIP1_STREAM = PAPERCLIP_STREAM % 3
PAPERCLIP2_STREAM = PAPERCLIP_STREAM % 4
PAPERCLIP3_STREAM = PAPERCLIP_STREAM % 5

PAPERCLIP_OBJ = """\
<<
    /Type /XObject
    /Subtype /Form
    /FormType 1
    /Length %d
    /Matrix [1.0 0.0 0.0 1.0 0.0 0.0]
    /Resources << /ProcSet [/PDF] >>
    /BBox [0.0 0.0 100.0 100.0]
>>
stream
%s
endstream
"""

PAPERCLIP1_OBJ = PAPERCLIP_OBJ % (len(PAPERCLIP1_STREAM), PAPERCLIP1_STREAM)
PAPERCLIP2_OBJ = PAPERCLIP_OBJ % (len(PAPERCLIP2_STREAM), PAPERCLIP2_STREAM)
PAPERCLIP3_OBJ = PAPERCLIP_OBJ % (len(PAPERCLIP3_STREAM), PAPERCLIP3_STREAM)
PAPERCLIPS = (pdftypes.PDFRef(PAPERCLIP1_ID), pdftypes.PDFRef(PAPERCLIP2_ID),
        pdftypes.PDFRef(PAPERCLIP3_ID))


class StaticObjects(object):
    def __init__(self, info, catalog, pages, pagelabels, outlines, resources,
            fonts, paperclips, lastOutline):
        self.info = info
        self.catalog = catalog
        self.pages = pages
        self.pagelabels = pagelabels
        self.outlines = outlines
        self.resources = resources
        self.fonts = fonts
        self.paperclips = paperclips
        self.lastOutline = lastOutline


def genPaperclips():
    paperclip1Obj = pdfobjs.PDFSimpleObject(PAPERCLIP1_ID, 0, PAPERCLIP1_OBJ)
    paperclip2Obj = pdfobjs.PDFSimpleObject(PAPERCLIP2_ID, 0, PAPERCLIP2_OBJ)
    paperclip3Obj = pdfobjs.PDFSimpleObject(PAPERCLIP3_ID, 0, PAPERCLIP3_OBJ)
    paperclipObjs = [paperclip1Obj, paperclip2Obj, paperclip3Obj]
    return paperclipObjs


def genStaticObjects(wf):
    # create all objects which should exist in the initial pdf file
    infoObj = pdfadvobjs.PDFInfoObject(INFO_ID, 0, wf.PDFTitle,
            wf.PDFAuthor, wf.PDFSubject, wf.PDFCreator, wf.PDFKeywords,
            wf.PDFProducer)
    catalogObj = pdfobjs.PDFSimpleObject(CATALOG_ID, 0, CATALOG_OBJ)
    pagesObj = pdfadvobjs.PDFPagesObject(PAGES_ID, 0)
    pagelabelsObj = pdfadvobjs.PDFPageLabelsObject(PAGELABELS_ID, 0)
    outlinesObj = pdfadvobjs.PDFOutlinesObject(OUTLINES_ID, 0)
    
    fontObjs = resources.genFonts()
    resourcesObj = resources.genBaseResources(RESOURCES_ID, 0)
    paperclipObjs = genPaperclips()
    
    objs = StaticObjects(info=infoObj, catalog=catalogObj, pages=pagesObj,
            pagelabels=pagelabelsObj, outlines=outlinesObj,
            resources=resourcesObj, fonts=fontObjs, paperclips=paperclipObjs,
            lastOutline=None)
    return objs

