"""
Module containing unittests for pdftypes.py.



This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: test_pdftypes.py 1071 2008-05-25 18:58:08Z ahuesgen $
"""

import pdftypes, pdfobjs
import unittest, verbosetestcase

class TestPDFNull(verbosetestcase.VerboseTestCase):
    def setUp(self):
        self.refNull = pdftypes.PDFNull()
    
    def tearDown(self):
        pass
    
    def testEqual(self):
        # test against normal null
        null = pdftypes.PDFNull()
        self.assertEquals(null, self.refNull)
        self.assertEquals(self.refNull, null)
        
        # test against modified null
        modNull = pdftypes.PDFNull()
        modNull.foo = "bar"
        self.assertEquals(modNull, self.refNull)
        self.assertEquals(self.refNull, modNull)
    
    def testUnequal(self):
        testData = [0, 1234, 0.0, "", "foo", None]
        
        for notNull in testData:
            msg = str(notNull)
            self.assertNotEquals(notNull, self.refNull, msg)
            self.assertNotEquals(self.refNull, notNull, msg)

class TestPDFNull(verbosetestcase.VerboseTestCase):
    def setUp(self):
        self.ref = pdftypes.PDFRef(1, 0)
    
    def tearDown(self):
        pass
    
    def testConstructor(self):
        testData = [(1, 0), (2, 0), (3, 0), (0, 1), (0, 2), (0, 3), (1, 1),
                    (123, 456)]
        
        for pair in testData:
            msg = str(pair)
            ref = pdftypes.PDFRef(pair[0], pair[1])
            self.assertEquals(ref.id, pair[0], msg)
            self.assertEquals(ref.gen, pair[1], msg)
    
    def testEqual(self):
        # test against equal reference
        ref = pdftypes.PDFRef(1, 0)
        self.assertEquals(ref, self.ref)
        self.assertEquals(self.ref, ref)
        
        # test against modified reference
        modRef = pdftypes.PDFRef(1, 0)
        modRef.foo = "bar"
        self.assertEquals(modRef, self.ref)
        self.assertEquals(self.ref, modRef)
        
        # test against derived object
        derivedRef = pdfobjs.PDFObject(1, 0)
        self.assertEquals(derivedRef, self.ref)
        self.assertEquals(self.ref, derivedRef)
    
    def testUnequal(self):
        testData = [pdftypes.PDFRef(2, 0), pdfobjs.PDFObject(2, 0), 
                pdftypes.PDFRef(1, 1), pdfobjs.PDFObject(1, 1),
                (1, 0), None, "1 0 R", "", 1.0]
        
        for value in testData:
            msg = str(value)
            self.assertNotEquals(value, self.ref, msg)
            self.assertNotEquals(self.ref, value, msg)

if __name__ == "__main__":
    unittest.main()
