"""
Module containing vhist related constants like version numbers etc.



This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: vhistconsts.py 2861 2013-05-28 10:14:40Z ahuesgen $
"""

import os
import sys


# The string $ VHIST_VERSION (without space) is replaced by the
# build driver with the correct version string
VHISTTOOLS_VERSION_STRING = "1.84.0.3018 of Jun 28 2013"
#VHISTADD_VERSION_STRING = "vhistadd " + VHISTTOOLS_VERSION_STRING
#VHISTXL_VERSION_STRING = "vhistxl " + VHISTTOOLS_VERSION_STRING

VHIST_VERSION = "VHIST-1.00"
#VHIST_CREATOR = VHISTADD_VERSION_STRING

VHIST_PDF_VERSION = "1.0"

VERSION_TAG = "%%<--$%s-->" % VHIST_VERSION
SUPPORTED_VERSION_TAGS = (VERSION_TAG)

#CREATOR_TAG = "%%<--$VHIST_CREATOR %s-->" % VHIST_CREATOR

def getAppDir():
    return os.path.dirname(os.path.realpath(sys.argv[0]))


def getVersion():
    global VHISTTOOLS_VERSION_STRING
    # prevent replacement by release script by splitting the identifier
    unsetVersion = "$VHIST" + "_VERSION"
    if VHISTTOOLS_VERSION_STRING == unsetVersion:
        # repo-root/code/vhistify => repo-root
        vhistaddDir = os.path.join(getAppDir(), "..")
        import runprocess
        (ret, stdout, stderr) = runprocess.runProcess("svnversion %", 
                vhistaddDir)
        if ret == 0:
            VHISTTOOLS_VERSION_STRING = "developer version %s" % stdout.strip()
        else:
            VHISTTOOLS_VERSION_STRING = "unknown version"
    
    return VHISTTOOLS_VERSION_STRING


def getVhistaddVersion():
    return "vhistadd " + getVersion()


def getVhistXLVersion():
    return "vhistxl " + getVersion()


def getVhistCreatorVersion():
    return getVhistaddVersion()

