"""
A simple XML parser and a related XML utility. 


This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: xmlutils.py 1071 2008-05-25 18:58:08Z ahuesgen $
"""

import xml.dom.minidom

class XML1Parser:
    """A simple class for parsing a simple XML file using a DOM approach.
        usage: 
            >>> xp = XML1Parser(myXMLString)
            >>> # set to required level
            >>> xp.setRootAbsolute(tag1="level1", tag2="level2")
            >>> # return data of element <myTag> at that level
            >>> xp.data("myTag")
            >>> # return list of elements
            >>> nodes = xp.getElementList("myTag")
            >>> # iterate over nodes
            >>> for node in nodes: print xp.data_t(nodes, "SomeTagInSubtree")

    """
    def __init__(self, szXML):
        self.elements = dict()
        self.dom = xml.dom.minidom.parseString(szXML)
        self.root = self.dom.documentElement
    
    def setRootAbsolute(self, tag1 = "", tag2 = "", tag3 = "", tag4 = ""):
        self.root = self.dom.documentElement
        try:
            if tag1 != "":
                self.root = self.root.getElementsByTagName(tag1)[0]
            if tag2 != "":
                self.root = self.root.getElementsByTagName(tag2)[0]
            if tag3 != "":
                self.root = self.root.getElementsByTagName(tag3)[0]
            if tag4 != "":
                self.root = self.root.getElementsByTagName(tag4)[0]
        except Exception, e:
            self.root = self.dom.documentElement
            return False
        return True
    
    def setRootNode(self, node):
        self.root = node
    
    def data(self, tag, default="undefined"):
        """
        returns data of element <tag> at current root level
        
        """
        try:
            node = self.root.getElementsByTagName(tag)[0]
            return node.firstChild.data
        except Exception, e:
            return default
    
    def data_t(self, troot, tag, default="undefined"):
        """
        returns data of element <tag> of subtree located below <troot>
        
        """
        try:
            node = troot.getElementsByTagName(tag)[0]
            return node.firstChild.data
        except Exception, e: 
            return default
    
    def getRootTagList(self):
        """
        returns a list of tag names located immediately below current root
        
        """
        taglist = []
        for node in self.root.childNodes:
            if node.nodeType == node.ELEMENT_NODE:
                taglist.append(node.tagName)
            return taglist
    
    def getElementList(self, tag):
        """
        returns a list of nodes with tag <tag> located immediately below
        current root
        
        """
        try:
            return self.root.getElementsByTagName(tag)
        except Exception, e: 
            return []

def xmlify8(xmlraw):
    """returns a string in UTF-8 encoding suitable for tag data"""
    xmlclean = xmlraw.replace("<", "&lt;")
    xmlclean = xmlclean.replace(">", "&gt;")
    return xmlclean.encode("utf-8")
