"""
TODO: update!
The module ``vhistblock`` contains the class %VhistBlock, which provides
a convenience wrapper around the classes %%vhistsection.VhistSection,
%%tempdir.TempDir and %%logger.Logger. %VhistBlock
has been designed to be used with python's ``with``-statement. You can write::

    from pyvhist.vhistblock import VhistBlock
    
    vhistfile = "output.vhist"
    with VhistBlock(vhistfile) as vb:
        doStuff()

It automatically creates a %%vhistsection.VhistSection object,
replaces stdout and
stderr with a logger, which automatically adds the programs output to the
workflow step, catches and records exceptions thrown by your code
and records the time spent inside the ``with`` code block. It also creates
a %%tempdir.TempDir instance, which will be used by the block to create
temporary files.
"""

import os

import vhistsection
from vhistsection import getCommandline, getHostname, getUser, getTool, \
        getToolpath
import vhistfile
import tempdir
import stopwatch
import logger
import iomonitor

import traceback

class VhistBlockResult(object):
    def __init__(self, section, tempDir, stopwatch):
        self.section = section
        self.tempDir = tempDir
        self.stopwatch = stopwatch

class VhistBlockIO(object):
    """
    """
    def __init__(self, title=None, description=None, comment=None,
            command=getCommandline, host=getHostname, user=getUser,
            tool=getTool, toolpath=getToolpath):
        """ Creates a vhist block.
        
        ``title``, ``description``, ``comment``, ``command``, ``host``,
        ``user``, ``tool`` and ``toolpath`` are the same as the arguments of
        the %%vhistsection.VhistSection constructor. These keyword arguments
        will be forwarded to the created workflow step.
        """
        
        #self.vhistFilename = vhistFilename
        #self.rootFilename = rootFilename
        self.section = vhistsection.VhistSection(title=title,
                description=description, comment=comment, command=command,
                host=host, user=user, tool=tool, toolpath=toolpath)
        self.section.userAttrs["cwd"] = os.getcwdu()
        self.tempdir = tempdir.TempDir()
        self.stopwatch = stopwatch.StopWatch()
        self.iom = iomonitor.IOMonitor(self.section)
        self.logger = logger.Logger(filename="output.log",
                tempDir=self.tempdir)
    
    def __enter__(self):
        td = self.tempdir.__enter__()
        sw = self.stopwatch.__enter__()
        log = self.logger.__enter__()
        iom = self.iom.__enter__()
        vlog = vhistfile.OutFile(log.filename, description="stdout and "
                "stderr output of the python process")
        self.section.files.append(vlog)
        
        result = VhistBlockResult(self.section, td, sw)
        return result
    
    def __exit__(self, type, value, tb):
        if type is not None:
            self.section.userAttrs["exception.type"] = type
            self.section.userAttrs["exception.value"] = value
            tb = "\n".join(traceback.format_tb(tb))
            self.section.userAttrs["exception.traceback"] = tb
        
        self.iom.__exit__(type, value, traceback)
        self.logger.__exit__(type, value, traceback)
        self.stopwatch.__exit__(type, value, traceback)
        self.stopwatch.addToSection(self.section)
        
        if len(self.iom.vhistOutfiles) == 0:
            self.iom.vhistOutfiles = ["untitled.vhist"]
        if self.iom.rootFilename is not None:
            self.section.createVhistSection(self.iom.vhistOutfiles,
                    self.iom.rootFilename)
        else:
            self.section.appendToVhistFile(self.iom.vhistOutfiles[0],
                    self.iom.vhistOutfiles[1:])
        self.tempdir.__exit__(type, value, traceback)

