
"""
Runs the given commandline call with strace. 

"""

import sys
import os
import signal

import utils
from runprocess import runProcess

from pyvhist import stopwatch 

class StraceError(Exception):
    pass


def readOutput(process, outputFilename):
    outputFile = open(outputFilename, "wb")
    try:
        readMore = True
        while readMore:
            byte = process.stdout.read(1)
            readMore = byte != "" and byte is not None
            if readMore:
                sys.stdout.write(byte)
                outputFile.write(byte)
                sys.stdout.flush()
    except IOError as e:
        pass
    outputFile.close()


def genInterruptHandler(retList, proc):
    def handleKeyboardInterrupt(signum, frame):
        retList.append(-signum)
    return handleKeyboardInterrupt


def findStraceExecutable():
    appDir = utils.getAppDir()
    straceExecutable = os.path.join(appDir, "strace")
    if utils.isExecutable(straceExecutable):
        return straceExecutable
    else:
        return utils.findProgram("strace")


def runCommand(args, tmpdir, forwardStdin=True):
    if len(args) == 0:
        raise StraceError("No command specified.")
    straceExecutable = findStraceExecutable()
    
    cwd = os.getcwd()
    tracefile = tmpdir.createTempFile("trace.txt")
    utils.createRestrictedFile(tracefile)
    
    mask = "% -xx -o % -f -e trace=open,creat,chdir,rename,execve -- " \
            + ("% " * len(args))
    
    with stopwatch.StopWatch() as watch:
        p = runProcess(mask, straceExecutable, tracefile, *args,
                mergeOutputs=True, returnProcessObject=True,
                forwardStdin=forwardStdin)
        if not forwardStdin:
            p.stdin.close()
        retList = []
        interruptHandler = genInterruptHandler(retList, p)
        signal.signal(signal.SIGINT, interruptHandler)
        
        outputFilename = tmpdir.createTempFile("stdout.log")
        utils.createRestrictedFile(outputFilename)
        readOutput(p, outputFilename)
        
        retList.append(p.wait())
        retval = retList[0]
        if retval < 0:
            # handle signals
            sigName = utils.getSignalNameById(-retval)
            retval = "%d (signal %s)" % (retval, sigName)
    
    signal.signal(signal.SIGINT, signal.SIG_DFL)
    
    tEnd = watch.endTime
    tStart = watch.startTime
    diff = [t2 - t1 for t1, t2 in zip(tStart, tEnd)]
    timeDiff = dict(user=diff[2], system=diff[3], elapsed=diff[4])
    print ""
    print "* returnvalue is", retval
    print "* current working directory is", cwd
    
    return retval, outputFilename, timeDiff, tracefile

