
import shlex, subprocess

# examples:
# 1)
#   (ret, out, err) = runProcess("svn diff")
# 
# 2)
#   (ret, out, err) = runProcess("rm -rf %", filename)
# 
# 3)
#   (ret, out, err) = runProcess("dd if=% of=%", inFilename, outFilename)
# 
# 4)
#   (ret, out) = runProcess("mkdir %", dirname, mergeOutputs=True)
# 
# 5)
#   (ret, out, err) = runProcess("grep foobar", stdin=someText)
# 
# 6)
#   (ret, _, _) = runProcess("make", printOutput=True)
# 
# 7)
#   processes = []
#   for ip in ips:
#       p = runProcess("ping -c 3 %", ip, returnProcessObject=True)
#       processes.append(p)
#
#   for ip, process in zip(ips, processes):
#       retval = p.wait()
#       print ip, retval
#
# 8)
#   call = "git add " + "% " * len(filenames) # e.g. git add % % %
#   (ret, out, err) = runProcess(call, *filenames)

def runProcess(command, *args, **kargs):
    (mergeOutputs, inputString, forwardStdin, cwd, printOutput, 
            returnProcessObject) = __parseArguments(kargs)
    splitCommand = shlex.split(command)
    realCommand = extendCommand(splitCommand, args)
    
    if forwardStdin:
        stdin = None
    else:
        stdin = subprocess.PIPE
    if printOutput:
        stdout = None
        stderr = None
    else:
        stdout = subprocess.PIPE
        stderr = subprocess.PIPE if not mergeOutputs else subprocess.STDOUT
    
    p = subprocess.Popen(realCommand, stdin=stdin, stdout=stdout,
            stderr=stderr, cwd=cwd, bufsize=0)
    
    if returnProcessObject:
        if inputString is not None and forwardStdin is False:
            p.stdin.write(inputString)
        return p
    else:
        if forwardStdin is False:
            (output, errout) = p.communicate()
        else:
            (output, errout) = p.communicate(inputString)
        retval = p.wait()
        
        if mergeOutputs:
            return (retval, output)
        else:
            return (retval, output, errout)

def extendCommand(arguments, replacements):
    restReplacements = list(replacements)
    finishedArgs = []
    for arg in arguments:
        splitarg = arg.split("%")
        numSplits = len(splitarg)
        if len(restReplacements) >= numSplits - 1:
            arg = interweave(splitarg, restReplacements[:numSplits - 1])
            del restReplacements[:numSplits - 1]
            arg = [str(x) for x in arg]
            arg = "".join(arg)
            finishedArgs.append(arg)
        else:
            errMessage = "not enough additional arguments"
            raise ValueError(errMessage)
    
    if len(restReplacements) > 0:
        errMessage = "too many additional arguments"
        raise ValueError(errMessage)

    return finishedArgs

def interweave(x, y):
    out = []
    i = 0
    while len(x) > i or len(y) > i:
        if len(x) > i:
            out.append(x[i])
        if len(y) > i:
            out.append(y[i])
        i += 1
    return out

def __parseArguments(dict):
    supportedArguments = ["mergeOutputs", "stdin", "cwd", "printOutput",
            "returnProcessObject", "forwardStdin"]
    unsupported = list(set(dict.keys()) - set(supportedArguments))
    if len(unsupported) > 0:
        err = "keyword(s) %s are not supported by runProcess()" % \
                ", ".join(unsupported)
        raise ValueError(err)
    
    mergeOutputs = dict.get("mergeOutputs", False)
    stdin = dict.get("stdin", None)
    forwardStdin = dict.get("forwardStdin", False)
    cwd = dict.get("cwd", None)
    printOutput = dict.get("printOutput", False)
    returnProcessObject = dict.get("returnProcessObject", False)
    return (mergeOutputs, stdin, forwardStdin, cwd, printOutput,
            returnProcessObject)

