
import sys
import os

import utils


def _tput(valueType):
    # tput seems to be very delicate.
    # 1) os.popen(tput...) returns the correct value but may print stderr to
    #    the shell (not good)
    # 2) subprocess.Popen(["tput...]) always returns 80 (no shell)
    # 3) subprocess.Popen(...shell=True) behaves like os.popen
    # 4) subprocess.Popen(...shell=True, stderr=PIPE) return 80 (WTF!!!)
    # 5) os.popen3(tput...) generates a deprection waring and returns 80
    # 6) os.popen(python getTerminalSize) and getTerminalSize reads $COLUMNS)
    # => we choose 1) and check that tput is available. Not perfect but better
    # than nothing. The problem is probably that a tput with two pipes
    # (stdout/stderr) does not create a virtual terminal
    
    if valueType not in ["lines", "cols"]:
        raise ValueError("valueType must be either 'lines' or 'cols'")
    
    if sys.platform in ["linux2", "sunos5", "darwin"]:
        # will throw an exception if tput is not in the path
        utils.findProgram("tput")
        # tput may write to stderr. How can we solve this? popen3 always
        # returns 80...
        stdout = os.popen("tput %s" % valueType)
        output = stdout.read().strip()
        return int(output)


def _readEnvShell(valueType):
    if valueType not in ["LINES", "COLUMNS"]:
        raise Exception("valueType must be either 'LINES' or 'COLUMNS'")
    
    if sys.platform in ["linux2", "sunos5", "darwin"]:
        stdout = os.popen("echo $%s" % valueType)
        output = stdout.read().strip()
        return int(output)


def _readEnvPython(valueType):
    if valueType not in ["LINES", "COLUMNS"]:
        raise Exception("valueType must be either 'LINES' or 'COLUMNS'")
        return int(os.environ[valueType])


def getNumColumns():
    try:
        return _tput("cols")
    except Exception:
        raise
    
    try:
        return _readEnvShell("COLUMNS")
    except Exception:
        pass
    
    try:
        return _readEnvPython("COLUMNS")
    except Exception:
        pass
    
    return 80


def getNumLines():
    try:
        return _tput("lines")
    except Exception:
        pass
    
    try:
        return _readEnvShell("LINES")
    except Exception:
        pass
    
    try:
        return _readEnvPython("LINES")
    except Exception:
        pass
    
    return 24

