
import sys
import os
import stat

class ProgramError(Exception):
    pass


def getAppDir():
    return os.path.dirname(os.path.realpath(sys.argv[0]))


def getLogDir():
    return os.path.join(getAppDir(), "logs")


def getPluginDir():
    return os.path.join(getAppDir(), "plugins")


def readFile(filename):
    with open(filename, "rb") as f:
        return f.read()


def createRestrictedFile(filename):
    """ Creates the given file if it does not exist and sets
        permissions to 600 """
    if not os.path.exists(filename):
        with open(filename, "wb") as f:
            pass
    os.chmod(filename, stat.S_IRUSR | stat.S_IWUSR)


def saveSyscalls(syscalls, tmpdir):
    filename = tmpdir.createTempFile("syscalls.txt")
    createRestrictedFile(filename)
    with open(filename, "wb") as logfile:
        for c in syscalls:
            logfile.write(c.toString() + "\n")


def saveFoundFiles(files, tmpdir):
    filename = tmpdir.createTempFile("foundfiles.txt")
    createRestrictedFile(filename)
    with open(filename, "wb") as logfile:
        for f in files:
            logfile.write(f.toString() + "\n")


def savePrograms(programs, tmpdir):
    filename = tmpdir.createTempFile("foundprograms.txt")
    createRestrictedFile(filename)
    with open(filename, "wb") as logfile:
        for p in programs:
            logfile.write(str(p) + "\n")


def getSignalNameById(sid):
    signals = {1: "SIGHUP", 2: "SIGINT", 3: "SIGQUIT", 4: "SIGILL", 5: "SIGTRAP",
            6: "SIGABRT", 7: "SIGBUS", 8: "SIGFPE", 9: "SIGKILL", 10: "SIGUSR1",
            11: "SIGSEGV", 12: "SIGUSR2", 13: "SIGPIPE", 14: "SIGALRM", 15: "SIGTERM"}
    return signals.get(sid, "???")


def isExecutable(programpath):
    return os.path.isfile(programpath) and os.access(programpath, os.X_OK)


def findProgram(programname):
    pathEnv = os.environ["PATH"]
    pathList = pathEnv.split(os.pathsep)
    for path in pathList:
        fullpath = os.path.realpath(os.path.join(path, programname))
        if isExecutable(fullpath):
            return fullpath
    raise ProgramError(("Could not find the program %s in path. This program "
            "is needed to run vhistify.") % programname)


def joinStrings(alist, separator=', ', lastSeparator=' and '):
    if len(alist) == 0:
        return ""
    elif len(alist) == 1:
        return alist[0]
    else:
        firstPart = separator.join(alist[:-1])
        lastPart = alist[-1]
        return firstPart + lastSeparator + lastPart

