
import os
from datetime import datetime

from pyvhist import vhistfile


def createVFiles(files):
    vfiles = []
    for finfo in files:
        if not os.path.isfile(finfo.filename):
            continue
        if "out" in finfo.types and "in" in finfo.types:
            vfile1 = vhistfile.OutFile(finfo.filename, optional=True)
            vfile2 = vhistfile.InFile(finfo.filename, optional=True,
                    comment="WARNING: probably modified by the program",
                    embed=False)
            vfiles.extend([vfile1, vfile2])
        elif "in" in finfo.types:
            vfile = vhistfile.InFile(finfo.filename, optional=True)
            vfiles.append(vfile)
        elif "out" in finfo.types:
            vfile = vhistfile.OutFile(finfo.filename, optional=True)
            vfiles.append(vfile)
    
    return vfiles


def addVhistFiles(vfiles):
    vvfiles = []
    rootfile = None
    for vfile in vfiles:
        vvfilename = vfile.filename + ".vhist"
        if vfile.isInfile() and os.path.isfile(vvfilename):
            if rootfile is None:
                rootfile = vvfilename
            else:
                vvfile = vhistfile.InFile(vvfilename, optional=True,
                        embed=True)
                vvfiles.append(vvfile)
        vvfiles.append(vfile)
    return vvfiles, rootfile


def _createMovedFilename(oldFilename, date):
    datestr = date.strftime("%Y-%m-%d_%H-%M-%S")
    dirname, basename = os.path.split(oldFilename)
    basename, extension = os.path.splitext(basename)
    newBasename = basename + ("-moved-on-%s" % datestr) + extension
    return os.path.join(dirname, newBasename)


def _renameFile(oldFilename, date):
    newFilename = _createMovedFilename(oldFilename, date)
    os.rename(oldFilename, newFilename)


def _sameFile(file1, file2):
    if file1 is None or file2 is None:
        return False
    else:
        return os.path.samefile(file1, file2)


def getOutputVhistFiles(vfiles, info, rootfile):
    outputVhistFiles = []
    for vfile in vfiles:
        if vfile.isOutfile():
            vvfilename = vfile.filename + ".vhist"
            outputVhistFiles.append(vvfilename)
    
    # if rootfile is in output files, move it to the first position
    if rootfile in outputVhistFiles:
        outputVhistFiles.remove(rootfile)
        outputVhistFiles.insert(0, rootfile)
    
    if len(outputVhistFiles) == 0:
        vvfilename = os.path.join(info.cwd, "untitled.vhist")
        outputVhistFiles.append(vvfilename)
    
    if info.renameOldVhistFiles:
        date = datetime.now()
        for outputVhistFile in outputVhistFiles:
            if os.path.exists(outputVhistFile):
                if not _sameFile(outputVhistFile, rootfile):
                    _renameFile(outputVhistFile, date)
    
    return outputVhistFiles

