
import baseplugin
import os

class FslPlugin(baseplugin.BasePlugin):
    """
    Enhances the output of FSL jobs.
    """
    
    def filterFiles(self, files):
        # try to remove all files in the $FSLDIR directory
        fslDir = os.environ.get("FSLDIR", None)
        if fslDir is None:
            return files
        else:
            fslDir = os.path.realpath(fslDir) + "/"
            files = [f for f in files if not f.filename.startswith(fslDir)]
            return files
    
    def amendFiles(self, files):
        # re-add all files in the $FSLDIR/data directory
        fslDir = os.environ.get("FSLDIR", None)
        if fslDir is None:
            return []
        else:
            fslDataDir = os.path.join(os.path.realpath(fslDir), "data/")
            
            files = [f for f in files if f.filename.startswith(fslDataDir)]
            return files
    
    def convertFiles(self, vfiles):
        fslDir = os.environ.get("FSLDIR", None)
        if fslDir is None:
            return []
        else:
            fslDataDir = os.path.join(os.path.realpath(fslDir), "data/")
            
            for f in vfiles:
                if f.filename.startswith(fslDataDir):
                    f.embed = False
            return vfiles
    
    def updateSection(self, section):
        fslExecutables = []
        for realProgram, program in self.infoObj.programs:
            if "fsl" in realProgram.lower():
                if realProgram not in fslExecutables:
                    fslExecutables.append(realProgram)
        
        if len(fslExecutables) > 0:
            section.userAttrs["fslExecutables"] = fslExecutables
        else:
            section.userAttrs["fslExecutables"] = \
                    "could not detect fsl executable"
        
        fslDir = os.environ.get("FSLDIR", None)
        if fslDir is not None:
            fslDir = os.path.abspath(fslDir)
            fslDir = self.userattrsModule.UserValue(fslDir, verbatim=True)
            section.userAttrs["FSLDIR"] = fslDir
            
        return section
    
    @classmethod
    def getDocumentation(cls):
        return DOCUMENTATION


DOCUMENTATION = """
The plugin documents the paths of the FSL tools used by your compute
job as well as the FSLDIR environment variable. The FSL plugin also
removes files which are part of the FSL installation from the VHIST
files as these are considered static. Files from FSL's data directory
are mentioned (but not embedded) into the VHIST files. This requires
that the FSLDIR environment variable is set correctly (see the FSL
documentation for details).
"""

