
import os

import baseplugin

MB = 1024 ** 2
MAX_FILESIZE_MB = 100
MAX_FILESIZE = MAX_FILESIZE_MB * MB

def isMD5File(filename):
    if not os.path.isfile(filename):
        return False
    else:
        with open(filename, "rb") as f:
            content = f.read()
            lines = content.splitlines()
            lines = [l.strip() for l in lines if l.strip() != ""]
            lines = [l.lower() for l in lines]
            if len(lines) != 1: # more than one line
                return False
            elif len(lines[0]) != 32: # not a 32 char md5
                return False
            elif any(x not in "0123456789abcdef" for x in lines[0]): # wrong char
                return False
            else:
                return True


class LargeFilePlugin(baseplugin.BasePlugin):
    """
    Disables MD5 checksums for very large files and reads MD5 sums from
    .md5 files on the file system.
    """
    def __init__(self):
        pass
    
    def convertFiles(self, files):
        for vfile in files:
            filename = vfile.filename
            md5file = filename + ".md5"
            if os.path.getsize(filename) > MAX_FILESIZE:
                vfile.automd5 = False
            if isMD5File(md5file):
                vfile.md5file = md5file
        
        return files
    
    @classmethod
    def getDocumentation(cls):
        return DOCUMENTATION


DOCUMENTATION = """
The "largefile" plugin disables the generation of MD5 checksums for files
larger than %d MB. If a file with the same name as the input/output file
and the extension .md5 exists on your file system (i.e. your output file
is called output.txt and a file output.txt.md5 is located in the same
directory), vhistify will try to read an MD5 sum from this file and
include it in the generated VHIST file(s). The file must not contain
anything but the MD5 checksum as an hexadecimal ascii string. You can
use this feature in case that your program generates an MD5 checksum on
it's own and you want to recycle this checksum.
""" % MAX_FILESIZE_MB

