
import os

import baseplugin
import zipsource

def isPythonFile(filename):
    return filename.endswith(".py") or filename.endswith(".pyc") or \
            filename.endswith(".pyo")

class PythonPlugin(baseplugin.BasePlugin):
    """
    Enhances the output of Python jobs.
    """
    def initEnvironment(self):
        # fixes problems with interactive scripts (python buffers output
        # when written into a pipe)
        os.environ["PYTHONUNBUFFERED"] = "Y"
    
    def convertFiles(self, files):
        pythonFiles = [f for f in files if isPythonFile(f.filename)]
        normalFiles = [f for f in files if not isPythonFile(f.filename)]
        
        zipfilename = self.tmpdir.createTempFile("sourcecode.zip")
        zipsource.zipSource(pythonFiles, zipfilename)
        vzip = self.vhistfileModule.InFile(zipfilename,
                description="python sourcecode", embed=True)
        
        return [vzip] + normalFiles
    
    def updateSection(self, section):
        pythonExecutables = []
        for realProgram, program in self.infoObj.programs:
            if "python" in os.path.basename(realProgram):
                pythonExecutables.append(realProgram)
        
        if len(pythonExecutables) > 0:
            section.userAttrs["pythonExecutables"] = pythonExecutables
        else:
            section.userAttrs["pythonExecutables"] = \
                    "could not detect python executable"
        
        return section
    
    @classmethod
    def getDocumentation(cls):
        return DOCUMENTATION


DOCUMENTATION = """
The Python plugin enhances the generated VHIST files for Python scripts.
If you enable this plugin, vhistify will try to detect the Python executable
used, disable buffering of standard output (necessary for some interactive
Python scripts that read from standard input) and pack all .py, .pyc and .pyo
input files (e.g. files read by the Python interpreter) into a sourcecode
archive.
"""

