
import re
import baseplugin

regex = "VHISTIFY: (?P<key>[a-zA-Z0-9.]+)(\\[(?P<unit>[^\\]]+)\\])?=(?P<value>.*)"


class StdoutAttributesPlugin(baseplugin.BasePlugin):
    """
    Reads key value pairs from the processes stdout and includes these as
    user-defined attributes in the generated VHIST file.
    """
    def updateSection(self, section):
        logContents = self.infoObj.logfileContents
        lines = logContents.splitlines()

        for line in lines:
            found = re.match(regex, line)
            if found:
                key = found.group("key")
                value = found.group("value")
                unit = found.group("unit")
                if unit is None:
                    value = self.userattrsModule.UserValue(value,
                            verbatim=True)
                else:
                    value = self.userattrsModule.UserValue(value, unit=unit,
                            verbatim=True)
                section.userAttrs[key] = value
        return section
    
    @classmethod
    def getDocumentation(cls):
        return DOCUMENTATION


DOCUMENTATION = """
The "stdoutattrs" plugin searches standard output and standard error for
lines for the form "VHISTIFY: key=value" of "VHISTIFY: key[unit]=value".
It splits these lines up into their components and includes the
keys and values as user-defined attributes in the generated VHIST files.
"""

